// SETA43 
// xsetaseta@gmail.com
// rejoj con sonido
// 28/05/2017
// Basado en  http://playground.arduino.cc/Main/DS1302
// PCD8544    http://playground.arduino.cc/Code/PCD8544
//            http://www.microsyl.com/index.php/2010/03/24/nokia-lcd-library/


#include <SoftwareSerial.h>

#include <avr/io.h>
#include <avr/interrupt.h>


#define DEBUG 
#define DIRECTORIO  5


#define DS1302_SCLK 15    // Arduino pin for the Serial Clock
#define DS1302_IO   16    // Arduino pin for the Data I/O
#define DS1302_CE   17    // Arduino pin for the Chip Enable


#define DS1302_SECONDS           0x80
#define DS1302_MINUTES           0x82
#define DS1302_HOURS             0x84
#define DS1302_DATE              0x86
#define DS1302_MONTH             0x88
#define DS1302_DAY               0x8A
#define DS1302_YEAR              0x8C
#define DS1302_ENABLE            0x8E
#define DS1302_TRICKLE           0x90
#define DS1302_CLOCK_BURST       0xBE
#define DS1302_CLOCK_BURST_WRITE 0xBE
#define DS1302_CLOCK_BURST_READ  0xBF
#define DS1302_RAMSTART          0xC0
#define DS1302_RAMEND            0xFC
#define DS1302_RAM_BURST         0xFE
#define DS1302_RAM_BURST_WRITE   0xFE
#define DS1302_RAM_BURST_READ    0xFF

#define DS1302_D0 0
#define DS1302_D1 1
#define DS1302_D2 2
#define DS1302_D3 3
#define DS1302_D4 4
#define DS1302_D5 5
#define DS1302_D6 6
#define DS1302_D7 7

#define DS1302_READBIT DS1302_D0 // READBIT=1: read instruction
#define DS1302_RC DS1302_D6
#define DS1302_CH DS1302_D7   // 1 = Clock Halt, 0 = start
#define DS1302_AM_PM DS1302_D5 // 0 = AM, 1 = PM
#define DS1302_12_24 DS1302 D7 // 0 = 24 hour, 1 = 12 hour

#define DS1302_WP DS1302_D7   // 1 = Write Protect, 0 = enabled

#define DS1302_ROUT0 DS1302_D0
#define DS1302_ROUT1 DS1302_D1
#define DS1302_DS0   DS1302_D2
#define DS1302_DS1   DS1302_D2
#define DS1302_TCS0  DS1302_D4
#define DS1302_TCS1  DS1302_D5
#define DS1302_TCS2  DS1302_D6
#define DS1302_TCS3  DS1302_D7


typedef struct ds1302_struct
{
  uint8_t Seconds:4;      // low decimal digit 0-9
  uint8_t Seconds10:3;    // high decimal digit 0-5
  uint8_t CH:1;       // CH = Clock Halt
  
  uint8_t Minutes:4;
  uint8_t Minutes10:3;
  uint8_t reserved1:1;
  
  union
  {
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:2;
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 0 for 24 hour format
    } h24;
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:1;
      uint8_t AM_PM:1;      // 0 for AM, 1 for PM
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 1 for 12 hour format
    } h12;
  };
  uint8_t Date:4;
  uint8_t Date10:2;
  uint8_t reserved3:2;
  
  uint8_t Month:4;
  uint8_t Month10:1;
  uint8_t reserved4:3;
  
  uint8_t Day:3;
  uint8_t reserved5:5;
  
  uint8_t Year:4;
  uint8_t Year10:4;
  
  uint8_t reserved6:7;
  uint8_t WP:1;             // WP = Write Protect
};

 ds1302_struct rtc;


#define E1PIN 8
#define E2PIN 9

byte horas,minutos,segundos,segundos2;
void printHora();
char buffer[80];     // the code uses 70 characters.

unsigned char DATASEND[10]=   {0x7E, 0xFF, 0x06, 0 ,0, 0, 0, 0, 0, 0xEF};
void SendData(unsigned char comando,int dato1);
int directorio;
SoftwareSerial mySerial(10,11); // RX, TX


unsigned int time1;
#define TIMER1  500

 void setup()
 {  
#ifdef DEBUG      
   Serial.begin(9600);
#endif  
   mySerial.begin(9600);

   pinMode(E1PIN, INPUT);
   pinMode(E2PIN, INPUT);

   DS1302_write (DS1302_ENABLE, 0);
   DS1302_write (DS1302_TRICKLE, 0x00);

//RESET - DS1312  -pulsar los dos pulsadores mientras arranca el reloj
if(!digitalRead(E1PIN) && !digitalRead(E2PIN)) resetChip();

#ifdef DEBUG 
   Serial.println("RELOJ SOUND");
#endif  
   LcdInitialise();
   PanIni();

   directorio=DIRECTORIO*256;
   SendData(12,0);// Reset CHIP MP3
   delay(1000);
 //  SendData(11,0);// Normal CHIP MP3
   SendData(6,22); //Volumen a 20
   delay(3000);
   SendData(15,directorio+1);
}


void loop()
{
  delay(1);
  time1++;
  if(time1>TIMER1)
  {
    time1=0;
    DS1302_clock_burst_read( (uint8_t *) &rtc); 
    segundos2=(rtc.Seconds10 * 10) + rtc.Seconds;
    minutos=(rtc.Minutes10 * 10) + rtc.Minutes;
    horas=(rtc.h24.Hour10 * 10) + rtc.h24.Hour;  
    if(segundos!=segundos2)
      {
       segundos=segundos2;
       printHora();
       if(segundos==0)
        {
           switch(minutos)
              {
              case 0: 
                if(horas>12)horas=horas-12;
#ifdef DEBUG                  
                Serial.print("Hora-");Serial.println(horas);
#endif                
                 SendData(15,directorio+horas);       
              break;
              case 15: 
                SendData(15,directorio+13); 
#ifdef DEBUG                  
                Serial.println("Cuartos");
#endif                
              break;
              case 30: 
                SendData(15,directorio+14);
#ifdef DEBUG                  
                Serial.println("Medias");
#endif                
              break;
              case 45: 
                SendData(15,directorio+15);
#ifdef DEBUG                  
                Serial.println("TresCuartos");
#endif                
              break;
              }
        }
      }

    
  }
      if(!digitalRead(E1PIN))
             {
#ifdef DEBUG                
              Serial.println("+minutos");
#endif              
              minutos++;
              segundos=0;
              if(minutos>59)minutos=0;
              menuTime();  
              delay(100);         
              //while(!digitalRead(E1PIN));
              delay(100);
              
             }
  
      if(!digitalRead(E2PIN))
             {
#ifdef DEBUG               
              Serial.println("+horas");
#endif              
              horas++;
              minutos=0;
              segundos=0;
              if(horas>23)horas=0; 
              menuTime();
              delay(100);
              //while(!digitalRead(E2PIN));             
              delay(100);
             }
}

//------------------------------------------
void resetChip()
{
  memset ((char *) &rtc, 0, sizeof(rtc));
  rtc.Seconds = 0;
  rtc.Seconds10 = 0;
  rtc.CH = 0;        // 1 for Clock Halt, 0 to run;
  rtc.Minutes = 0;
  rtc.Minutes10 = 0;
  rtc.h24.Hour = 0;
  rtc.h24.Hour10 = 0;
  rtc.h24.hour_12_24 = 0; // 0 for 24 hour format
  rtc.Date = 1;
  rtc.Date10 = 0;
  rtc.Month = 1;
  rtc.Month10 = 0;
  rtc.Day = 1;
  rtc.Year = 1;
  rtc.Year10 = 1;
  rtc.WP = 0;  
  DS1302_clock_burst_write( (uint8_t *) &rtc);
}
void PanIni()
{

  LcdClear();
  drawBox(); 
  gotoXY(0,0);
  LcdString("Reloj SOUND"); 
}  

void menuTime()
{
 byte h1,h,m1,m;
  h1=horas/10; h=horas-(h1*10);
  h1=h1<<4; h1=h1 | h;
  DS1302_write(0x84,h1);
  
  m1=minutos/10; m=minutos-(m1*10);
  m1=m1<<4; m1=m1 | m;
  DS1302_write(0x82,m1);
  
  DS1302_write(0x80,0); //Clear segundos
  printHora();
}

void printHora()
{
#ifdef DEBUG  
     sprintf(buffer,"%02d:%02d:%02d",horas,minutos,segundos); 
     Serial.println(buffer);
#endif     
     sprintf(buffer,"%02d:%02d",horas,minutos); 
     gotoXY(8,2);
     LcdStringX(buffer);
     sprintf(buffer,"%02d",segundos); 
     gotoXY(24,4);
     LcdStringX(buffer);

}

//_______________________________________


void SendData(unsigned char comando,int dato1)
{
 int z;
 unsigned int suma;

  DATASEND[3]=comando;
  DATASEND[5]=dato1>>8;
  DATASEND[6]=dato1 & 0xFF;
  suma=DATASEND[1]+DATASEND[2]+DATASEND[3]+DATASEND[4]+DATASEND[5]+DATASEND[6];
  suma=0-suma;
  DATASEND[7]=suma>>8;
  DATASEND[8]=suma & 0x00FF;
 
  for(z=0;z<10;z++)
    {
       mySerial.write(DATASEND[z]);
#ifdef DEBUG       
     Serial.print(DATASEND[z],HEX ); Serial.print(" ");      
#endif       
    }   
#ifdef DEBUG     
    Serial.println(" ");
#endif     
}

//--------------------------------------

void DS1302_clock_burst_read( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_READ, true);  
  for (i=0; i<8; i++)    *p++ = _DS1302_toggleread();  
  _DS1302_stop();
}


void DS1302_clock_burst_write( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_WRITE, false);  
  for (i=0; i<8; i++)    _DS1302_togglewrite( *p++, false);  
  _DS1302_stop();
}

uint8_t DS1302_read(int address)
{
  uint8_t data;

  bitSet (address, DS1302_READBIT);  
  _DS1302_start();
  _DS1302_togglewrite (address, true);  
  data = _DS1302_toggleread ();
  _DS1302_stop();

  return (data);
}

void DS1302_write(int address, uint8_t data)
{
  bitClear (address, DS1302_READBIT);  

  _DS1302_start();
  _DS1302_togglewrite (address, false);
  _DS1302_togglewrite (data, false);
  _DS1302_stop();  
}

void _DS1302_start(void)
{
  digitalWrite (DS1302_CE, LOW); // default, not enabled
  pinMode (DS1302_CE, OUTPUT);  
  digitalWrite (DS1302_SCLK, LOW); // default, clock low
  pinMode (DS1302_SCLK, OUTPUT);
  pinMode (DS1302_IO, OUTPUT);
  digitalWrite (DS1302_CE, HIGH); // start the session
  delayMicroseconds(4);           // tCC = 4us
}

void _DS1302_stop(void)
{
  digitalWrite (DS1302_CE, LOW);
  delayMicroseconds(4);           // tCWH = 4us
}


uint8_t _DS1302_toggleread(void)
{
  uint8_t i, data;

  data = 0;
  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_SCLK, HIGH);
    delayMicroseconds(1);
    digitalWrite (DS1302_SCLK, LOW);
    delayMicroseconds(1);        // tCL=1000ns, tCDD=800ns
    bitWrite (data, i, digitalRead(DS1302_IO));
  }
  return (data);
}


void _DS1302_togglewrite(uint8_t data, uint8_t release)
{
  int i;

  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_IO, bitRead(data, i));  
    delayMicroseconds(1);     // tDC = 200ns
    digitalWrite (DS1302_SCLK, HIGH);    
    delayMicroseconds(1);     // tCH = 1000ns, tCDH = 800ns
    if (release && i == 7)
    {
      pinMode (DS1302_IO, INPUT);
      digitalWrite (DS1302_IO, LOW); // remove any pull-up  
    }
    else
    {
      digitalWrite (DS1302_SCLK, LOW);
      delayMicroseconds(1);       // tCL=1000ns, tCDD=800ns
    }
  }
}


//___________________________________________



// The pins to use on the arduino
#define PIN_SCE   3
#define PIN_RESET 4
#define PIN_DC    5
#define PIN_SDIN  6
#define PIN_SCLK  7 
 
// COnfiguration for the LCD
#define LCD_C     LOW
#define LCD_D     HIGH
#define LCD_CMD   0
 
// Size of the LCD
#define LCD_X     84
#define LCD_Y     48
 
int scrollPosition = -10;
 
static const byte ASCII[][5] =
{
 {0x00, 0x00, 0x00, 0x00, 0x00} // 20
,{0x00, 0x00, 0x5f, 0x00, 0x00} // 21 !
,{0x00, 0x07, 0x00, 0x07, 0x00} // 22 "
,{0x14, 0x7f, 0x14, 0x7f, 0x14} // 23 #
,{0x24, 0x2a, 0x7f, 0x2a, 0x12} // 24 $
,{0x23, 0x13, 0x08, 0x64, 0x62} // 25 %
,{0x36, 0x49, 0x55, 0x22, 0x50} // 26 &
,{0x00, 0x05, 0x03, 0x00, 0x00} // 27 '
,{0x00, 0x1c, 0x22, 0x41, 0x00} // 28 (
,{0x00, 0x41, 0x22, 0x1c, 0x00} // 29 )
,{0x14, 0x08, 0x3e, 0x08, 0x14} // 2a *
,{0x08, 0x08, 0x3e, 0x08, 0x08} // 2b +
,{0x00, 0x50, 0x30, 0x00, 0x00} // 2c ,
,{0x08, 0x08, 0x08, 0x08, 0x08} // 2d -
,{0x00, 0x60, 0x60, 0x00, 0x00} // 2e .
,{0x20, 0x10, 0x08, 0x04, 0x02} // 2f /
,{0x3e, 0x51, 0x49, 0x45, 0x3e} // 30 0
,{0x00, 0x42, 0x7f, 0x40, 0x00} // 31 1
,{0x42, 0x61, 0x51, 0x49, 0x46} // 32 2
,{0x21, 0x41, 0x45, 0x4b, 0x31} // 33 3
,{0x18, 0x14, 0x12, 0x7f, 0x10} // 34 4
,{0x27, 0x45, 0x45, 0x45, 0x39} // 35 5
,{0x3c, 0x4a, 0x49, 0x49, 0x30} // 36 6
,{0x01, 0x71, 0x09, 0x05, 0x03} // 37 7
,{0x36, 0x49, 0x49, 0x49, 0x36} // 38 8
,{0x06, 0x49, 0x49, 0x29, 0x1e} // 39 9
,{0x00, 0x36, 0x36, 0x00, 0x00} // 3a :
,{0x00, 0x56, 0x36, 0x00, 0x00} // 3b ;
,{0x08, 0x14, 0x22, 0x41, 0x00} // 3c <
,{0x14, 0x14, 0x14, 0x14, 0x14} // 3d =
,{0x00, 0x41, 0x22, 0x14, 0x08} // 3e >
,{0x02, 0x01, 0x51, 0x09, 0x06} // 3f ?
,{0x32, 0x49, 0x79, 0x41, 0x3e} // 40 @
,{0x7e, 0x11, 0x11, 0x11, 0x7e} // 41 A
,{0x7f, 0x49, 0x49, 0x49, 0x36} // 42 B
,{0x3e, 0x41, 0x41, 0x41, 0x22} // 43 C
,{0x7f, 0x41, 0x41, 0x22, 0x1c} // 44 D
,{0x7f, 0x49, 0x49, 0x49, 0x41} // 45 E
,{0x7f, 0x09, 0x09, 0x09, 0x01} // 46 F
,{0x3e, 0x41, 0x49, 0x49, 0x7a} // 47 G
,{0x7f, 0x08, 0x08, 0x08, 0x7f} // 48 H
,{0x00, 0x41, 0x7f, 0x41, 0x00} // 49 I
,{0x20, 0x40, 0x41, 0x3f, 0x01} // 4a J
,{0x7f, 0x08, 0x14, 0x22, 0x41} // 4b K
,{0x7f, 0x40, 0x40, 0x40, 0x40} // 4c L
,{0x7f, 0x02, 0x0c, 0x02, 0x7f} // 4d M
,{0x7f, 0x04, 0x08, 0x10, 0x7f} // 4e N
,{0x3e, 0x41, 0x41, 0x41, 0x3e} // 4f O
,{0x7f, 0x09, 0x09, 0x09, 0x06} // 50 P
,{0x3e, 0x41, 0x51, 0x21, 0x5e} // 51 Q
,{0x7f, 0x09, 0x19, 0x29, 0x46} // 52 R
,{0x46, 0x49, 0x49, 0x49, 0x31} // 53 S
,{0x01, 0x01, 0x7f, 0x01, 0x01} // 54 T
,{0x3f, 0x40, 0x40, 0x40, 0x3f} // 55 U
,{0x1f, 0x20, 0x40, 0x20, 0x1f} // 56 V
,{0x3f, 0x40, 0x38, 0x40, 0x3f} // 57 W
,{0x63, 0x14, 0x08, 0x14, 0x63} // 58 X
,{0x07, 0x08, 0x70, 0x08, 0x07} // 59 Y
,{0x61, 0x51, 0x49, 0x45, 0x43} // 5a Z
,{0x00, 0x7f, 0x41, 0x41, 0x00} // 5b [
,{0x02, 0x04, 0x08, 0x10, 0x20} // 5c ¥
,{0x00, 0x41, 0x41, 0x7f, 0x00} // 5d ]
,{0x04, 0x02, 0x01, 0x02, 0x04} // 5e ^
,{0x40, 0x40, 0x40, 0x40, 0x40} // 5f _
,{0x00, 0x01, 0x02, 0x04, 0x00} // 60 `
,{0x20, 0x54, 0x54, 0x54, 0x78} // 61 a
,{0x7f, 0x48, 0x44, 0x44, 0x38} // 62 b
,{0x38, 0x44, 0x44, 0x44, 0x20} // 63 c
,{0x38, 0x44, 0x44, 0x48, 0x7f} // 64 d
,{0x38, 0x54, 0x54, 0x54, 0x18} // 65 e
,{0x08, 0x7e, 0x09, 0x01, 0x02} // 66 f
,{0x0c, 0x52, 0x52, 0x52, 0x3e} // 67 g
,{0x7f, 0x08, 0x04, 0x04, 0x78} // 68 h
,{0x00, 0x44, 0x7d, 0x40, 0x00} // 69 i
,{0x20, 0x40, 0x44, 0x3d, 0x00} // 6a j
,{0x7f, 0x10, 0x28, 0x44, 0x00} // 6b k
,{0x00, 0x41, 0x7f, 0x40, 0x00} // 6c l
,{0x7c, 0x04, 0x18, 0x04, 0x78} // 6d m
,{0x7c, 0x08, 0x04, 0x04, 0x78} // 6e n
,{0x38, 0x44, 0x44, 0x44, 0x38} // 6f o
,{0x7c, 0x14, 0x14, 0x14, 0x08} // 70 p
,{0x08, 0x14, 0x14, 0x18, 0x7c} // 71 q
,{0x7c, 0x08, 0x04, 0x04, 0x08} // 72 r
,{0x48, 0x54, 0x54, 0x54, 0x20} // 73 s
,{0x04, 0x3f, 0x44, 0x40, 0x20} // 74 t
,{0x3c, 0x40, 0x40, 0x20, 0x7c} // 75 u
,{0x1c, 0x20, 0x40, 0x20, 0x1c} // 76 v
,{0x3c, 0x40, 0x30, 0x40, 0x3c} // 77 w
,{0x44, 0x28, 0x10, 0x28, 0x44} // 78 x
,{0x0c, 0x50, 0x50, 0x50, 0x3c} // 79 y
,{0x44, 0x64, 0x54, 0x4c, 0x44} // 7a z
,{0x00, 0x08, 0x36, 0x41, 0x00} // 7b {
,{0x00, 0x00, 0x7f, 0x00, 0x00} // 7c |
,{0x00, 0x41, 0x36, 0x08, 0x00} // 7d }
,{0x10, 0x08, 0x08, 0x10, 0x08} // 7e ←
,{0x00, 0x06, 0x09, 0x09, 0x06} // 7f →
};
 
void LcdCharacter(char character)
{
  unsigned char z,z1;
  
  z1=character - 0x20;
  LcdWrite(LCD_D, 0x00);
  for (int index = 0; index < 5; index++)
  {
    //para que funciona en proteus  
    z=ASCII[z1][index];
    LcdWrite(LCD_D, z);
   // LcdWrite(LCD_D, ASCII[character - 0x20][index]);    
  }
  LcdWrite(LCD_D, 0x00);
}
void LcdCharacterX(char character)
{
  unsigned char z,z1;
   z1=character - 0x20;       
  LcdWrite(LCD_D, 0x00);
  for (int index = 0; index < 5; index++)
  {
     //para que funciona en proteus
    z=ASCII[z1][index];
    LcdWrite(LCD_D, z);
    LcdWrite(LCD_D, z);
    //LcdWrite(LCD_D, ASCII[character - 0x20][index]);
    //LcdWrite(LCD_D, ASCII[character - 0x20][index]);
  }
  LcdWrite(LCD_D, 0x00);
}  
void LcdClear(void)
{
  for (int index = 0; index < LCD_X * LCD_Y / 8; index++)
  {
    LcdWrite(LCD_D, 0x00);
  }
}
 
void LcdInitialise(void)
{
  pinMode(PIN_SCE,   OUTPUT);
  pinMode(PIN_RESET, OUTPUT);
  pinMode(PIN_DC,    OUTPUT);
  pinMode(PIN_SDIN,  OUTPUT);
  pinMode(PIN_SCLK,  OUTPUT);
 
  digitalWrite(PIN_RESET, LOW);
  digitalWrite(PIN_RESET, HIGH);
 
  LcdWrite(LCD_CMD, 0x21);  // LCD Extended Commands.
  LcdWrite(LCD_CMD, 0xBf);  // Set LCD Vop (Contrast). //B1
  LcdWrite(LCD_CMD, 0x04);  // Set Temp coefficent. //0x04
  LcdWrite(LCD_CMD, 0x14);  // LCD bias mode 1:48. //0x13
  LcdWrite(LCD_CMD, 0x0C);  // LCD in normal mode. 0x0d for inverse
  LcdWrite(LCD_C, 0x20);
  LcdWrite(LCD_C, 0x0C);
}
 
void LcdString(char *characters)
{
  while (*characters)
  {
    LcdCharacter(*characters++);
  }
}
void LcdStringX(char *characters)
{
  while (*characters)
  {
    LcdCharacterX(*characters++);
  }
} 
void LcdWrite(byte dc, byte data)
{
  digitalWrite(PIN_DC, dc);
  digitalWrite(PIN_SCE, LOW);
  shiftOut(PIN_SDIN, PIN_SCLK, MSBFIRST, data);
  digitalWrite(PIN_SCE, HIGH);
}
 
/**
 * gotoXY routine to position cursor
 * x - range: 0 to 84
 * y - range: 0 to 5
 */
void gotoXY(int x, int y)
{
  LcdWrite( 0, 0x80 | x);  // Column.
  LcdWrite( 0, 0x40 | y);  // Row.
}
 
void drawBox(void)
{
  int j;
  for(j = 0; j < 84; j++) // top
  {
    gotoXY(j, 0);
    LcdWrite(1, 0x01);
  }     
 
  for(j = 0; j < 84; j++) //Bottom
  {
    gotoXY(j, 5);
    LcdWrite(1, 0x80);
  }     
 
  for(j = 0; j < 6; j++) // Right
  {
    gotoXY(83, j);
    LcdWrite(1, 0xff);
  }     
 
  for(j = 0; j < 6; j++) // Left
  {
    gotoXY(0, j);
    LcdWrite(1, 0xff);
  }
}
 
void Scroll(String message)
{
  for (int i = scrollPosition; i < scrollPosition + 11; i++)
  {
    if ((i >= message.length()) || (i < 0))
    {
      LcdCharacter(' ');
    }
    else  
    {
      LcdCharacter(message.charAt(i));
    }
  }
  scrollPosition++;
  if ((scrollPosition >= message.length()) && (scrollPosition > 0))
  {
    scrollPosition = -10;
  }
}



//___________________________________________

